#import "RNMetrica.h"

#import <YandexMobileMetrica/YandexMobileMetrica.h>
#import <CoreLocation/CoreLocation.h>

@implementation RNMetrica {
    BOOL gYMMIsAppMetricaActivated;
}

RCT_EXPORT_MODULE();

- (dispatch_queue_t)methodQueue {
    return dispatch_queue_create("com.facebook.React.AsyncLocalStorageQueue", DISPATCH_QUEUE_SERIAL);
}

RCT_EXPORT_METHOD(activate:(NSDictionary *)configurationDictionary)
{
    [[self class] activateWithConfigurationDictionary:configurationDictionary];
}

RCT_REMAP_METHOD(isActivated,
                 isActivatedWithResolver:(RCTPromiseResolveBlock)resolve
                 rejecter: (RCTPromiseRejectBlock)reject)
{
    return resolve(self.gYMMIsAppMetricaActivated);
}

RCT_EXPORT_METHOD(handleOpenURL:(NSString *)urlString)
{
    NSURL *url = [NSURL URLWithString:urlString];
    if (url != nil) {
        [YMMYandexMetrica handleOpenURL:URL];
    }
}

RCT_EXPORT_METHOD(reportEvent:(NSString *)eventName value:(NSDictionary *)eventParams callback:(RCTResponseSenderBlock)callback)
{
    [YMMYandexMetrica reportEvent:eventName
                      parameters:eventParams
                        onFailure:^(NSError *error) {
                            NSDictionary *jsError = RCTMakeError(@"Failed to report event", error, nil);
                            callback(@[@[jsError]]);
                        }];
}

RCT_EXPORT_METHOD(reportError:(NSString *)errorName value:(NSString *)errorReason callback:(RCTResponseSenderBlock)callback)
{
    NSException *exception = [NSException exceptionWithName:errorName reason:errorReason userInfo:nil];
    [YMMYandexMetrica reportError:errorName
                      exception:exception
                        onFailure:^(NSError *error) {
                            NSDictionary *jsError = RCTMakeError(@"Failed to report error", error, nil);
                            callback(@[@[jsError]]);
                        }];
}

RCT_EXPORT_METHOD(setCustomAppVersion:(NSString *)appVersion)
{
    [YMMYandexMetrica setCustomAppVersion:appVersion];
}

RCT_EXPORT_METHOD(setLocation:(NSDictionary *)locationDictionary)
{
    CLLocation *location = [[self class] locationForDictionary:locationDictionary];
    [YMMYandexMetrica setLocation:location];
}

RCT_EXPORT_METHOD(setTrackLocationEnabled:(NSNumber *)enabledValue)
{
    if (enabledValue != nil) {
        [YMMYandexMetrica setTrackLocationEnabled:enabledValue.boolValue];
    }
}

RCT_EXPORT_METHOD(setEnvironmentValue:(NSString *)key value:(NSString *)value)
{
    [YMMYandexMetrica setEnvironmentValue:value forKey:key];
}

RCT_EXPORT_METHOD(setSessionTimeout:(NSNumber *)sessionTimeoutValue)
{
    if (sessionTimeoutValue != nil) {
        [YMMYandexMetrica setSessionTimeout:sessionTimeoutValue.unsignedIntegerValue];
    }
}

RCT_EXPORT_METHOD(setReportCrashesEnabled:(NSNumber *)enabledValue)
{
// Do nothing.
//    if (enabledValue != nil) {
//        [YMMYandexMetrica setReportCrashesEnabled:enabledValue.boolValue];
//    }
}

RCT_EXPORT_METHOD(setLoggingEnabled)
{
    [YMMYandexMetrica setLoggingEnabled:YES];
}

RCT_EXPORT_METHOD(setCollectInstalledAppsEnabled)
{
    // Do nothing. Not available for iOS platform.
}

#pragma mark - Utils

+ (YMMYandexMetricaConfiguration *)configurationForDictionary:(NSDictionary *)configurationDictionary
{
    NSString *apiKey = configurationDictionary[@"apiKey"];
    YMMYandexMetricaConfiguration *configuration = [[YMMYandexMetricaConfiguration alloc] initWithApiKey:apiKey];

    NSNumber *handleFirstActivationAsUpdateEnabled = configurationDictionary[@"handleFirstActivationAsUpdateEnabled"];
    NSNumber *trackLocationEnabled = configurationDictionary[@"trackLocationEnabled"];
    NSNumber *sessionTimeout = configurationDictionary[@"sessionTimeout"];
    NSNumber *reportCrashesEnabled = configurationDictionary[@"reportCrashesEnabled"];
    NSString *customAppVersion = configurationDictionary[@"appVersion"];
    NSNumber *loggingEnabled = configurationDictionary[@"loggingEnabled"];
    NSDictionary *customLocationDictionary = configurationDictionary[@"location"];
    NSDictionary *preloadInfoDictionary = configurationDictionary[@"preloadInfo"];

    if (handleFirstActivationAsUpdateEnabled != nil) {
        configuration.handleFirstActivationAsUpdateEnabled = [handleFirstActivationAsUpdateEnabled boolValue];
    }
    if (trackLocationEnabled != nil) {
        configuration.trackLocationEnabled = [trackLocationEnabled boolValue];
    }
    if (sessionTimeout != nil) {
        configuration.sessionTimeout = [sessionTimeout unsignedIntegerValue];
    }
    if (reportCrashesEnabled != nil) {
        configuration.reportCrashesEnabled = [reportCrashesEnabled boolValue];
    }
    if (customAppVersion != nil) {
        configuration.customAppVersion = customAppVersion;
    }
    if (loggingEnabled != nil) {
        configuration.loggingEnabled = [loggingEnabled boolValue];
    }
    if (customLocationDictionary != nil) {
        configuration.location = [self locationForDictionary:customLocationDictionary];;
    }
    if (preloadInfoDictionary != nil) {
        NSString *trackingID = preloadInfoDictionary[@"trackingId"];
        YMMYandexMetricaPreloadInfo *preloadInfo =
        [[YMMYandexMetricaPreloadInfo alloc] initWithTrackingIdentifier:trackingID];
        NSDictionary *additionalInfo = preloadInfoDictionary[@"additionalInfo"];
        for (NSString *key in additionalInfo) {
            [preloadInfo setAdditionalInfo:additionalInfo[key] forKey:key];
        }
        configuration.preloadInfo = preloadInfo;
    }

    return configuration;
}

+ (CLLocation *)locationForDictionary:(NSDictionary *)locationDictionary
{
    if (locationDictionary == nil) {
        return nil;
    }

    NSNumber *latitude = locationDictionary[@"latitude"];
    NSNumber *longitude = locationDictionary[@"longitude"];
    NSNumber *altitude = locationDictionary[@"altitude"];
    NSNumber *horizontalAccuracy = locationDictionary[@"accuracy"];
    NSNumber *verticalAccuracy = locationDictionary[@"verticalAccuracy"];
    NSNumber *course = locationDictionary[@"course"];
    NSNumber *speed = locationDictionary[@"speed"];
    NSNumber *timestamp = locationDictionary[@"timestamp"];

    NSDate *locationDate = timestamp != nil ? [NSDate dateWithTimeIntervalSince1970:timestamp.doubleValue] : [NSDate date];
    CLLocationCoordinate2D coordinate = CLLocationCoordinate2DMake(latitude.doubleValue, longitude.doubleValue);
    CLLocation *location = [[CLLocation alloc] initWithCoordinate:coordinate
                                                         altitude:altitude.doubleValue
                                               horizontalAccuracy:horizontalAccuracy.doubleValue
                                                 verticalAccuracy:verticalAccuracy.doubleValue
                                                           course:course.doubleValue
                                                            speed:speed.doubleValue
                                                        timestamp:locationDate];
    return location;
}

+ (void)activateWithConfigurationDictionary:(NSDictionary *)configuration
{
    YMMYandexMetricaConfiguration *config = [[self class] configurationForDictionary:configuration];
    [YMMYandexMetrica activateWithConfiguration:config];
    gYMMIsAppMetricaActivated = true;
}

@end
